import com.sun.java.swing.*;
import com.sun.java.swing.border.*;
import java.awt.*;
import java.awt.event.*;

/********Class Graph_Pane******/
/*
  this class contains the graph as well as the other components
  that either communicate with the graph or alter the graph.
  Components:
  1 Menu Bar 
    -- Graph representation menu
  2 Title on a panel
  3 Time Value display
  
  This class is also an event listener for these components
  To make Graph_Pane communicate the marked time to the top-level container,
  the container needs to be passed as an object to this class using the second
  constructor, the event code in the mouseClicked function needs to be changed 
  - add the necessary function to the container and utillize it in the if 
    statement, replace the (Graph_Pane) explicit typecasting with the
    (container class) type cast. which basically means that you need to also
    provide a double variable in the container called markd_time.
*/
public class Graph_Pane extends JPanel
implements MouseMotionListener,MouseListener,ActionListener
{
    Dimension dimensions;
    JPanel title;
    JLabel title_label;
    Graph graph;
    TimeInterface time_listener;
    JTextField track_time;
    
    Cursor default_cur = new Cursor(Cursor.DEFAULT_CURSOR),
	on_graph = new Cursor(Cursor.CROSSHAIR_CURSOR);

    
    /*
      Constructor
      if no container class specified then no probs except that it wont
      communicate the selected value on the x-axis.
    */
    public Graph_Pane
	(String label,int values[],double start_time,double end_time)
	
    {
	super(new GridBagLayout());
	GridBagConstraints constraints = new GridBagConstraints();
	constraints.fill = GridBagConstraints.NONE;
	constraints.anchor = GridBagConstraints.CENTER;

	JMenuBar menubar = new JMenuBar();
	menubar.setLayout(new FlowLayout(FlowLayout.LEFT,10,5));
	JMenu menu = new JMenu("Graph");
	menubar.add(menu);
	JMenuItem item = new JMenuItem("Bar");
	item.addActionListener(this);
	menu.add(item);
	item = new JMenuItem("Line");
	item.addActionListener(this);
	menu.add(item);
	menubar.setBorder(new EtchedBorder());
	constraints.gridx = 0;
	constraints.gridy = 0;
	add(menubar,constraints);
	
	title = new JPanel(new BorderLayout(0,0));
	title_label = new JLabel(label,JLabel.CENTER);
	title.add(title_label,BorderLayout.CENTER);
	constraints.gridx = 0;
	constraints.gridy = 1;
	add(title,constraints);

	JPanel temp_panel = new JPanel(new FlowLayout(FlowLayout.RIGHT,10,5));
	JLabel temp_label = new JLabel("Time:",JLabel.CENTER);
	temp_panel.add(temp_label);
	track_time = new JTextField("0.0",7);
	track_time.setEnabled(false);
	temp_panel.add(track_time);
	constraints.gridx = 0;
	constraints.gridy = 2;
	add(temp_panel,constraints);

	graph = new Graph(values,start_time, end_time);
	dimensions = graph.getDimension();	
	constraints.gridx = 0;
	constraints.gridy = 3;
	add(graph,constraints);    
	graph.addMouseMotionListener(this);
	graph.addMouseListener(this);
	
	temp_label.setBackground(Color.white);
	temp_panel.setBackground(Color.white);
	title_label.setBackground(Color.white);
	title.setBackground(Color.white);

	Dimension temp_dimension = new Dimension(dimensions.width,30);
	temp_panel.setPreferredSize(temp_dimension);
	temp_panel.setMaximumSize(temp_dimension);
	temp_panel.setMinimumSize(temp_dimension);
	title.setPreferredSize(temp_dimension);
	title.setMaximumSize(temp_dimension);
	title.setMinimumSize(temp_dimension);
	menubar.setPreferredSize(temp_dimension);
	menubar.setMaximumSize(temp_dimension);
	menubar.setMinimumSize(temp_dimension);
	dimensions.setSize(dimensions.width,
			   dimensions.height+20+30+30);
       	setPreferredSize(dimensions);
	setMaximumSize(dimensions);
	setMinimumSize(dimensions);
	setBackground(Color.white);
    }
    public void init()
    {
    }
    
    public Dimension getDimension()
    {
	return new Dimension(dimensions);
    }

    public void setTimeListener(TimeInterface ti)
    {
	time_listener = ti;
    }

    public void mousePressed(MouseEvent e)
    {
    }
    public void mouseReleased(MouseEvent e){}
    public void mouseEntered(MouseEvent e){}
    public void mouseExited(MouseEvent e){}

    public void mouseClicked(MouseEvent e)
    {
	if(time_listener != null)
	    {
		Point coords = e.getPoint(),
		    graph_origin = graph.getOrigin();
		if((coords.x >= graph_origin.x) && 
		   (coords.x <= (graph_origin.x + graph.getGraphRange())) &&
		   (coords.y <= graph_origin.y) &&
		   (coords.y > graph.graph_margin))
		    time_listener.showTime(graph.mark_time(coords)); 
	    }
    }

    public void mouseMoved(MouseEvent e)
    {
	Point coords = e.getPoint(),
	    graph_origin = graph.getOrigin();
	String time_value_s;

        if((coords.x >= graph_origin.x) && 
	   (coords.x <= (graph_origin.x + graph.getGraphRange())) &&
	   (coords.y <= graph_origin.y) &&
	   (coords.y > graph.graph_margin))
	    {
		graph.setCursor(on_graph);
		time_value_s = graph.calculateXValueString(coords.x,8);
		track_time.setText(time_value_s);
	    }
	else
	    graph.setCursor(default_cur);
		
    }
    public void mouseDragged(MouseEvent e){}
    public void actionPerformed(ActionEvent e)
    {
	if(e.getActionCommand().compareTo("Line") == 0)
	    graph.changeGraphType(Graph.LINE);
	else if(e.getActionCommand().compareTo("Bar") == 0)
	    graph.changeGraphType(Graph.BAR);
    }

	      
    
}
